#include <GL/glfw.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <time.h>
#include <string.h>
#include <math.h>

#include "portaudio.h"

#define rnd(n) (rand()%(n+1))

// fixed point "casts"
#define I2F(n) ((n)<<8)
#define F2I(n) ((n)>>8)

void LogPrint(char *string, ...)
{
	FILE *file;
	char temp[1024];
	va_list args;

	va_start(args, string);
	vsprintf(temp, string, args);
	va_end(args);

	file=fopen("log.txt", "a");
	fprintf(file,"\n%s", temp);
	fclose(file);
}

#ifdef WIN32
extern "C" long _ftol( double ); //defined by VC6 C libs
extern "C" long _ftol2( double dblSource ) { return _ftol( dblSource ); }
extern "C" long _ftol2_sse( double dblSource ) { return _ftol( dblSource ); }
#endif

bool editor=false;

// --- audio

struct Channel
{
	bool active;
	bool mute;
	float volume;
	int p;
	float period;
	int iperiod;
	float phase;
	int iphase;
	float slide;
	float pslide;
	float fade;
	bool noise;
	int noisep;
	float noiseval;
	
	int pindex;
	int pdelay;
	int ppos;
};

Channel channels[8];
int tick=0;

struct SCommand
{
	int op;
	int param;
};

struct SProgram
{
	int length;
	int speed;
	SCommand commands[1024];
};

SProgram sprograms[20];

float notef[120];

float master_volume=0.5f;

PortAudioStream *stream;

static int AudioCallback(void *inputBuffer, void *outputBuffer,
						 unsigned long framesPerBuffer,
						 PaTimestamp outTime, void *userData)
{
	(void)userData;
	(void)outTime;
	(void)inputBuffer;
	short *out=(short*)outputBuffer;

	for(int ch=0;ch<8;ch++)
		if(channels[ch].active)
		{
			channels[ch].iphase=(int)(channels[ch].period*channels[ch].phase);
			channels[ch].iperiod=(int)channels[ch].period;
		}
	for(unsigned int i=0;i<framesPerBuffer;i++)
	{
		float sample=0.0f;
		for(int ch=0;ch<8;ch++)
			if(channels[ch].active)
			{
				if(channels[ch].mute)
					continue;
				channels[ch].p++;
				if(channels[ch].p>channels[ch].iperiod) // wrap
				{
					channels[ch].p=0;
				}
				if(channels[ch].noise)
				{
					int noisep=channels[ch].p*8/channels[ch].iperiod;
					if(noisep!=channels[ch].noisep)
					{
						channels[ch].noisep=noisep;
						channels[ch].noiseval=(float)(rnd(16)-8)/8.0f*channels[ch].volume;
					}
					sample+=channels[ch].noiseval;
				}
				else
				{
					if(channels[ch].p<channels[ch].iphase)
						sample+=channels[ch].volume;
					else
						sample-=channels[ch].volume;
				}
			}

		(*out++)=(short)(sample*master_volume*32768);

		tick++;
		if((tick&63)==0) // update channel programs (effects, slides, music)
		{
			for(int ch=0;ch<8;ch++)
				if(channels[ch].active)
				{
					if(channels[ch].pindex>-1)
					{
						// program tick
						SProgram *prg=&sprograms[channels[ch].pindex];
						channels[ch].pdelay++;
	//					LogPrint(".");
						if(channels[ch].pdelay>=prg->speed)
						{
	//						LogPrint("trig");
							channels[ch].pdelay=9999; // trigger again almost immediately (unless delayed)
							channels[ch].ppos++;
							SCommand *cmd=&prg->commands[channels[ch].ppos];
							switch(cmd->op)
							{
							case 0x00: // delay
								channels[ch].pdelay=0;
								break;
							case 0x01: // start note
								channels[ch].period=44100.0f/notef[cmd->param];
	//							LogPrint("play (period=%i)", channels[ch].period);
								channels[ch].mute=false;
								break;
							case 0x02: // stop note
								channels[ch].mute=true;
								break;
							case 0x03: // set slide
								channels[ch].slide=(float)cmd->param/100;
								break;
							case 0x05: // set phase
								channels[ch].phase=(float)cmd->param/100;
								break;
							case 0x06: // set noise
								if(cmd->param==1)
									channels[ch].noise=true;
								else
									channels[ch].noise=false;
								break;
							case 0xFE: // jump to start
	//							LogPrint("end");
								channels[ch].ppos=-1;
								break;
							case 0xFF: // end program
	//							LogPrint("end");
								channels[ch].active=false;
								break;
							}
						}
					}
					
					channels[ch].phase+=channels[ch].pslide*0.001f;
					if(channels[ch].phase>0.95f)
						channels[ch].phase=0.95f;
					if(channels[ch].phase<0.05f)
						channels[ch].phase=0.05f;
					channels[ch].volume+=channels[ch].fade*0.001f;
					if(channels[ch].volume<0.0f)
						channels[ch].volume=0.0f;
					if(channels[ch].volume>1.0f)
						channels[ch].volume=1.0f;
					channels[ch].period*=1.0f+channels[ch].slide*0.1f;
					channels[ch].iphase=(int)(channels[ch].period*channels[ch].phase);
					channels[ch].iperiod=(int)channels[ch].period;
				}
		}
	}

	return 0;
}

void AddSCommand(SProgram& program, int op, int param)
{
	program.commands[program.length].op=op;
	program.commands[program.length].param=param;
	program.length++;
}

void PlaySound(int prg, int ch, float volume)
{
//	LogPrint("PlaySound()");
	channels[ch].mute=true;
	channels[ch].noise=false;
	channels[ch].noisep=0;
	channels[ch].noiseval=0.0f;
	channels[ch].volume=volume;
	channels[ch].phase=0.5f;
	channels[ch].slide=0.0f;
	channels[ch].pslide=0.0f;
	channels[ch].fade=0.0f;
	channels[ch].pindex=prg;
	channels[ch].ppos=-1;
	channels[ch].pdelay=9999; // trigger as soon as possible
	channels[ch].active=true;
}

// --- structs

struct Sprite
{
	GLuint texture;
	int width;
	int height;
};

struct Box
{
	int x;
	int y;
	int w;
	int h;
};

struct Machine
{
	bool exists;
	int x;
	int y;
	int fx;
	int fy;
	int dx;
	int dy;
	int oldfx;
	int oldfy;
	int dispfx;
	int dispfy;
	int tilemap[32*32];
	Box box;
	
	bool collided;
};

struct Player
{
	int x;
	int y;
	int fx;
	int fy;
	int dx;
	int dy;
	Box box;

	int detach;
	int airtime;
	bool onfloor;
	bool flipped;
	int thrust;

	int frame;
	int reload;
	int anim_index;
	int anim_pos;
	int anim_delay;
};

struct Star
{
	int x;
	int y;
	int type;
};

struct Particle // used for various things
{
	bool exists;
	int life;
	int x;
	int y;
	int fx;
	int fy;
	int dx;
	int dy;
	int frame;
	int delay;
};

// --- globals

Sprite tiles[100];
Machine machines[200];

Sprite spr_stars[4];
Star stars[2000];

int panim[10][20];

Sprite spr_player[10];
Player player;
Sprite spr_pshot;
Particle pshots[20];
Sprite spr_thrust[3];

Sprite spr_bpoff[3];
Particle bpoff[10];

Sprite spr_blip;
Sprite spr_mapgrid;
Sprite spr_win;

int collectibles=0;

int camera_x;
int camera_y;

bool gameover=false;
float death_alpha;

// ---

void LoadSprite(char *filename, Sprite& sprite)
{
	GLFWimage image;
	glfwReadImage(filename, &image, 0);
	// color key
	unsigned int *idata=(unsigned int*)malloc(image.Width*image.Height*4);
	int i=0;
	for(int y=0;y<image.Width;y++)
		for(int x=0;x<image.Height;x++)
		{
			unsigned int pixel=0x00000000;
			pixel|=image.Data[i*3+0];
			pixel|=image.Data[i*3+1]<<8;
			pixel|=image.Data[i*3+2]<<16;
			if(pixel!=0x00800080) // opaque
				pixel|=0xFF000000;
			idata[i++]=pixel;
		}
	sprite.width=image.Width;
	sprite.height=image.Height;
	glGenTextures(1, &sprite.texture);
//	LogPrint("image.width=%i image.height=%i image.texture=%i", sprite.width, sprite.height, sprite.texture);
	glBindTexture(GL_TEXTURE_2D, sprite.texture);
	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
	glTexParameterf(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
	gluBuild2DMipmaps(GL_TEXTURE_2D, 4, image.Width, image.Height, GL_RGBA, GL_UNSIGNED_BYTE, idata);
	free(idata);
	glfwFreeImage(&image);
}

void DrawSprite(Sprite& sprite, int x, int y, bool flip)
{
//	if(x>camera_x+160 || y>camera_y+120 || x+sprite.width<camera_x-160 || y+sprite.height<camera_y-120)
//		return;
	float xflip=0.0f;
	if(flip) xflip=1.0f;
	glBindTexture(GL_TEXTURE_2D, sprite.texture);
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f+xflip, 1.0f);
	glVertex3i(x, y, 0);
	glTexCoord2f(0.0f+xflip, 0.0f);
	glVertex3i(x, y+sprite.height, 0);
	glTexCoord2f(1.0f-xflip, 0.0f);
	glVertex3i(x+sprite.width, y+sprite.height, 0);
	glTexCoord2f(1.0f-xflip, 1.0f);
	glVertex3i(x+sprite.width, y, 0);
	glEnd();
}

void DrawSprite(Sprite& sprite, int x, int y)
{
	DrawSprite(sprite, x, y, false);
}

void DrawMachine(Machine& machine, bool map)
{
	for(int y=0;y<32;y++)
		for(int x=0;x<32;x++)
		{
			int t=machine.tilemap[y*32+x];
			if(map)
			{
				if(t==10) // collectible, draw on map
					DrawSprite(spr_blip, 16+(machine.x+x*16)/32, 16+(machine.y+y*16)/32);
			}
			else if(t>-1)
				DrawSprite(tiles[t], machine.x+x*16, machine.y+y*16);
		}
}

void DrawBox(Box& box)
{
	glDisable(GL_TEXTURE_2D);
	glBegin(GL_QUADS);
	glVertex3i(box.x, box.y, 0);
	glVertex3i(box.x, box.y+box.h, 0);
	glVertex3i(box.x+box.w, box.y+box.h, 0);
	glVertex3i(box.x+box.w, box.y, 0);
	glEnd();
	glEnable(GL_TEXTURE_2D);
}

// ---

void SLimit(int& val, int limit)
{
	if(val>limit)
		val=limit;
	if(val<-limit)
		val=-limit;
}

void FDecay(int& val, int speed)
{
	if(val>0)
	{
		val-=speed;
		if(val<0) val=0;
	}
	else
	{
		val+=speed;
		if(val>0) val=0;
	}
}

void SetBox(Box& box, int x, int y, int w, int h)
{
	box.x=x;
	box.y=y;
	box.w=w;
	box.h=h;
}

bool BoxIntersect(Box& b1, Box& b2)
{
	if(b2.x<b1.x+b1.w && b2.x+b2.w>b1.x && b2.y<b1.y+b1.h && b2.y+b2.h>b1.y)
		return true;
	return false;
}

void BlockPoff(int x, int y)
{
	for(int i=0;i<10;i++)
		if(!bpoff[i].exists)
		{
			bpoff[i].exists=true;
			bpoff[i].x=x;
			bpoff[i].y=y;
			bpoff[i].delay=5;
			bpoff[i].frame=0;
			break;
		}
}

void PlayerDie()
{
	if(!gameover)
	{
		PlaySound(5, 6, 0.2f);
		death_alpha=1.0f;
	}
	gameover=true;
}

void MachineSetTile(Machine& machine, int x, int y, int index)
{
	if(x>=0 && x<32 && y>=0 && y<32)
		machine.tilemap[y*32+x]=index;
}

int MachineGetTile(Machine& machine, int x, int y)
{
	if(x>=0 && x<32 && y>=0 && y<32)
		return machine.tilemap[y*32+x];
	return -1;
}

void GenerateMachine(Machine& machine, int type)
{
	for(int y=0;y<32;y++)
		for(int x=0;x<32;x++)
			machine.tilemap[y*32+x]=-1;

	char fname[32];
	sprintf(fname, "maps/map%i.dat", type);
	LogPrint("open \"%s\"", fname);
	FILE *file=fopen(fname, "rb");
	LogPrint("file=%i", file);
	fread(machine.tilemap, 32*32, sizeof(int), file);
	fclose(file);

	for(int y=0;y<32;y++)
		for(int x=0;x<32;x++)
			if(machine.tilemap[y*32+x]==10)
				collectibles++;
}

void MachineCastPos(Machine& machine)
{
	machine.x=F2I(machine.fx);
	machine.y=F2I(machine.fy);
}

void MachineMove(Machine& machine)
{
	machine.dispfx=machine.fx-machine.oldfx;
	machine.oldfx=machine.fx;
	machine.dispfy=machine.fy-machine.oldfy;
	machine.oldfy=machine.fy;
	machine.fx+=machine.dx;
	machine.fy+=machine.dy;
}

void MachineMakeBox(Machine& machine, bool size)
{
	machine.box.x=machine.x;
	machine.box.y=machine.y;
	if(size)
	{
		machine.box.w=0;
		machine.box.h=0;
		for(int y=0;y<32;y++)
			for(int x=0;x<32;x++)
			{
				int t=machine.tilemap[y*32+x];
				if(t>-1)
				{
					if(machine.box.w<x*16+16)
						machine.box.w=x*16+16;
					if(machine.box.h<y*16+16)
						machine.box.h=y*16+16;
				}
			}
	}
}

bool BoxMachineIntersect(Box& box, Machine& machine, int type)
{
	if(!BoxIntersect(box, machine.box))
		return false;
	int mdx=box.x-machine.x;
	int mdy=box.y-machine.y;
	int t2x=mdx/16;
	int t2y=mdy/16;
	for(int ty=t2y-1;ty<t2y+2;ty++)
		for(int tx=t2x-1;tx<t2x+2;tx++)
			if(tx>=0 && tx<32 && ty>=0 && ty<32)
			{
				int ti2=ty*32+tx;
				int t2=machine.tilemap[ti2];
				if(t2>-1 && t2!=10)
				{
					Box t2box;
					t2box.x=machine.x+tx*16;
					t2box.y=machine.y+ty*16;
					t2box.w=16;
					t2box.h=16;
					if(BoxIntersect(box, t2box))
					{
						if(type==1) // hit by shot
						{
							if(t2==6) // destructable block
							{
								BlockPoff(t2box.x, t2box.y);
								machine.tilemap[ti2]=-1;
								PlaySound(1, 1, 0.1f);
								PlaySound(2, 2, 0.1f);
							}
						}
						return true;
					}
				}
			}
	return false;
}

bool PlayerCollide(Machine& machine, bool attach)
{
	Box tempbox=player.box;
	tempbox.h+=4; // use larger box to cover possible floor-sticking
	if(!BoxIntersect(tempbox, machine.box))
		return false;
	int edgestates[4];
	for(int i=0;i<4;i++)
		edgestates[i]=0; // 0=no hit, 1=low priority hit, 2=definite hit
	int maxpriority=1;
	bool hit=false;
	int mdx=player.x-machine.x;
	int mdy=player.y-machine.y;
	int t2x=mdx/16;
	int t2y=mdy/16;
	for(int ty=t2y-1;ty<t2y+2;ty++)
		for(int tx=t2x-1;tx<t2x+2;tx++)
			if(tx>=0 && tx<32 && ty>=0 && ty<32)
			{
				int ti2=ty*32+tx;
				int t2=machine.tilemap[ti2];
				if(t2>-1)
				{
					Box t2box;
					t2box.x=machine.x+tx*16;
					t2box.y=machine.y+ty*16;
					t2box.w=16;
					t2box.h=16;
					if(BoxIntersect(player.box, t2box))
					{
						if(t2==10) // collectible
						{
							machine.tilemap[ti2]=-1;
							collectibles--;
							PlaySound(3, 4, 0.1f);
							continue;
						}
						hit=true;
						int dx=t2box.x-player.box.x;
						int dy=t2box.y-player.box.y;
						// check which edge was hit and with what priority
						int edge=0;
						// 1 - rotated quadrant, which edge
						if(abs(dx)>abs(dy))
						{
							if(dx>0)
								edge=0; // right
							else
								edge=1; // left
						}
						else
						{
							if(dy>0)
								edge=2; // down
							else
								edge=3; // up
						}
						// 2 - quadrant crossing, what priority
						int priority=1;
						if((edge==0 || edge==1) && abs(dy)<8)
							priority=2;
						if((edge==2 || edge==3) && abs(dx)<8)
							priority=2;
						// 3 - set state for this edge
						if(edgestates[edge]<priority)
							edgestates[edge]=priority;
						if(priority==2)
							maxpriority=2;
					}
				}
			}
	if(attach) // attach feet to floor
	{
		int tx1=(player.x+2-machine.x)/16;
		int tx2=(player.x+13-machine.x)/16;
		int ty=(player.y+17-machine.y)/16;
		int tile1=MachineGetTile(machine, tx1, ty);
		int tile2=MachineGetTile(machine, tx2, ty);
		if((tile1>-1 && tile1!=10) || (tile2>-1 && tile2!=10))
		{
			player.y=machine.y+(ty-1)*16;
			player.fy=I2F(player.y);
			player.dy=machine.dy;

			player.dx-=machine.dx;
			if(abs(player.dx)>256)
				FDecay(player.dx, 24);
			FDecay(player.dx, 1);
			player.dx-=player.dx/8;
			player.dx+=machine.dx;

//			player.fx+=machine.dispfx;

//			attach_x=machine.x+tx1*16;
//			attach_y=machine.y+ty*16;
			edgestates[2]=0; // don't do regular collisions with the down edge
			
			player.onfloor=true;
		}
	}
	if(hit)
	{
		int rdx=player.dx-machine.dispfx;
		int rdy=player.dy-machine.dispfy;
		if(maxpriority==2)
			for(int i=0;i<4;i++)
				edgestates[i]--; // remove all priority 1 hits
		if(edgestates[0]>0 && (rdx>0 || player.onfloor)) // right hit
		{
			int overlap=16-((1024+machine.x-player.x)%16);
			if(overlap>5) // squished
				PlayerDie();
			player.x-=overlap;
			player.dx=machine.dx;
			player.fx=I2F(player.x);
		}
		if(edgestates[1]>0 && (rdx<0 || player.onfloor)) // left hit
		{
			int overlap=((1024+machine.x-player.x)%16);
			if(overlap>5) // squished
				PlayerDie();
			player.x+=overlap;
			player.dx=machine.dx;
			player.fx=I2F(player.x);
		}
		if(edgestates[2]>0 && rdy>0) // down hit
		{
			int overlap=16-((1024+machine.y-player.y)%16);
			if(overlap>5) // squished
				PlayerDie();
			player.y-=overlap;
			player.dy=machine.dy;
			player.fy=I2F(player.y);
		}
		if(edgestates[3]>0 && rdy<0) // up hit
		{
			int overlap=((1024+machine.y-player.y)%16);
			if(overlap>5) // squished
				PlayerDie();
			player.y+=overlap;
			player.dy=machine.dy;
			player.fy=I2F(player.y);
		}
		return true;
	}
	return false;
}

bool MachinesCollide(Machine& m1, Machine& m2, bool move)
{
	if(!BoxIntersect(m1.box, m2.box))
		return false;
	int edgestates[4];
	for(int i=0;i<4;i++)
		edgestates[i]=0; // 0=no hit, 1=low priority hit, 2=definite hit
	int maxpriority=1;
	bool hit=false;
	for(int y=0;y<32;y++)
		for(int x=0;x<32;x++)
		{
			int ti1=y*32+x;
			int t1=m1.tilemap[ti1];
			if(t1>-1) // non-empty tile in first machine
			{
				Box t1box;
				t1box.x=m1.x+x*16;
				t1box.y=m1.y+y*16;
				t1box.w=16;
				t1box.h=16;
				int mdx=m2.x-m1.x;
				int mdy=m2.y-m1.y;
				int t2x=x-mdx/16;
				int t2y=y-mdy/16;
				for(int ty=t2y-1;ty<t2y+2;ty++)
					for(int tx=t2x-1;tx<t2x+2;tx++)
						if(tx>=0 && tx<32 && ty>=0 && ty<32)
						{
							int ti2=ty*32+tx;
							int t2=m2.tilemap[ti2];
							if(t2>-1)
							{
								Box t2box;
								t2box.x=m2.x+tx*16;
								t2box.y=m2.y+ty*16;
								t2box.w=16;
								t2box.h=16;
								if(BoxIntersect(t1box, t2box))
								{
									hit=true;
									int dx=t2box.x-t1box.x;
									int dy=t2box.y-t1box.y;
									// check which edge was hit and with what priority
									int edge=0;
									// 1 - rotated quadrant, which edge
									if(abs(dx)>abs(dy))
									{
										if(dx>0)
											edge=0; // right
										else
											edge=1; // left
									}
									else
									{
										if(dy>0)
											edge=2; // down
										else
											edge=3; // up
									}
									// 2 - quadrant crossing, what priority
									int priority=1;
									if((edge==0 || edge==1) && abs(dy)<8)
										priority=2;
									if((edge==2 || edge==3) && abs(dx)<8)
										priority=2;
									// 3 - set state for this edge
									if(edgestates[edge]<priority)
										edgestates[edge]=priority;
									if(priority==2)
										maxpriority=2;
								}
							}
						}
			}
		}
	if(hit)
	{
		if(!move)
			return true; // just report collision, don't react
		int m1fx_old=m1.fx;
		int m1fy_old=m1.fy;
		int m2fx_old=m2.fx;
		int m2fy_old=m2.fy;
		int dx=(m1.dx+m2.dx)/2;
		int dy=(m1.dy+m2.dy)/2;
		if(maxpriority==2)
			for(int i=0;i<4;i++)
				edgestates[i]--; // remove all priority 1 hits

		if(abs(m1.dx-m2.dx)>128) // heavy collision, horizontal
		{
//			for(hitpoints)
//				spawn_puffs();
		}
//		if(abs(m1.dy-m2.dy)>128) // heavy collision, vertical
		// ...
		
		if(edgestates[0]>0) // right hit
		{
			int overlap=16-((1024+m2.x-m1.x)%16);
			if(overlap>1)
			{
				m1.x-=overlap/2;
				m2.x+=overlap/2;
				m1.fx-=I2F(overlap)/2;
				m2.fx+=I2F(overlap)/2;
			}
			else
			{
				m1.x-=overlap;
				m1.fx-=I2F(overlap);
			}
			m1.dx=dx;
			m2.dx=dx;
//			m1.fx=I2F(m1.x);
//			m2.fx=I2F(m2.x);
		}
		if(edgestates[1]>0) // left hit
		{
			int overlap=((1024+m2.x-m1.x)%16);
			if(overlap>1)
			{
				m1.x+=overlap/2;
				m2.x-=overlap/2;
				m1.fx+=I2F(overlap)/2;
				m2.fx-=I2F(overlap)/2;
			}
			else
			{
				m1.x+=overlap;
				m1.fx+=I2F(overlap);
			}
			m1.dx=dx;
			m2.dx=dx;
			m1.fx=I2F(m1.x);
			m2.fx=I2F(m2.x);
		}
		if(edgestates[2]>0) // down hit
		{
			int overlap=16-((1024+m2.y-m1.y)%16);
			if(overlap>1)
			{
				m1.y-=overlap/2;
				m2.y+=overlap/2;
				m1.fy-=I2F(overlap)/2;
				m2.fy+=I2F(overlap)/2;
			}
			else
			{
				m1.y-=overlap;
				m1.fy-=I2F(overlap);
			}
			m1.dy=dy;
			m2.dy=dy;
			m1.fy=I2F(m1.y);
			m2.fy=I2F(m2.y);
		}
		if(edgestates[3]>0) // up hit
		{
			int overlap=((1024+m2.y-m1.y)%16);
			if(overlap>1)
			{
				m1.y+=overlap/2;
				m2.y-=overlap/2;
				m1.fy+=I2F(overlap)/2;
				m2.fy-=I2F(overlap)/2;
			}
			else
			{
				m1.y+=overlap;
				m1.fy+=I2F(overlap);
			}
			m1.dy=dy;
			m2.dy=dy;
			m1.fy=I2F(m1.y);
			m2.fy=I2F(m2.y);
		}
/*		m1.dx=m1.fx-m1fx_old;
		m1.dy=m1.fy-m1fy_old;
		m2.dx=m2.fx-m2fx_old;
		m2.dy=m2.fy-m2fy_old;*/
		return true;
	}
	return false;
}

void Reset()
{
	for(int i=0;i<200;i++)
		machines[i].exists=false;

	collectibles=0;

	if(!editor)
		for(int i=0;i<60;i++)
		{
			machines[i].exists=true;
			if(i==0)
				GenerateMachine(machines[i], 0);
			else
				GenerateMachine(machines[i], rnd(43));
			MachineMakeBox(machines[i], true);
			bool placed=false;
			while(!placed)
			{
				if(i==0)
				{
					machines[i].x=0;
					machines[i].y=0;
				}
				else
				{
					machines[i].x=rnd(2048);
					machines[i].y=rnd(2048);
				}
				MachineMakeBox(machines[i], false);
				placed=true;
				for(int j=0;j<200;j++)
					if(machines[j].exists && j!=i && MachinesCollide(machines[i], machines[j], false))
					{
						placed=false;
						break;
					}
			}
			machines[i].dx=0;
			machines[i].dy=0;
			machines[i].fx=I2F(machines[i].x);
			machines[i].fy=I2F(machines[i].y);
			machines[i].oldfx=machines[i].fx;
			machines[i].oldfy=machines[i].fy;
			machines[i].dispfx=0;
			machines[i].dispfy=0;
		}

	for(int i=0;i<20;i++)
		pshots[i].exists=false;

	for(int i=0;i<10;i++)
		bpoff[i].exists=false;

	player.x=64;
	player.y=-16;
	player.fx=I2F(player.x);
	player.fy=I2F(player.y);
	player.dx=0;
	player.dy=0;
	player.detach=0;
	player.frame=0;
	player.airtime=0;
	player.flipped=false;
	player.reload=0;
	player.anim_index=-1;
	
	camera_x=player.x-160;
	camera_y=player.y-120;
}

int main()
{
	srand(time(NULL));
	
	FILE *file=fopen("log.txt", "w");
	fprintf(file,"log started\n");
	fclose(file);

	glfwInit();

	if(!glfwOpenWindow(640, 480, 0,0,0,0, 0,0, GLFW_WINDOW))
	{
		glfwTerminate();
		return 0;
	}
	glfwSetWindowTitle("7th_swarming_of_the_machines");

	glfwSwapInterval(1);

	glfwEnable(GLFW_STICKY_KEYS);

	glViewport(0, 0, 640, 480);
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
//	glOrtho(0,320,240,0, 1.0f, -1.0f);
	float zoom=1.0f;
	glOrtho(160-zoom*160,160+zoom*160, 120+zoom*120,120-zoom*120, 1.0f, -1.0f);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	glEnable(GL_TEXTURE_2D);
	glColor4f(1.0f, 1.0f, 1.0, 1.0f);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	// init audio
	for(int i=0;i<5;i++)
		channels[i].active=false;
	
	// note frequencies
	notef[0]=55.00f;
	notef[1]=58.27f;
	notef[2]=61.74f;
	notef[3]=65.41f;
	notef[4]=69.30f;
	notef[5]=73.42f;
	notef[6]=77.78f;
	notef[7]=82.41f;
	notef[8]=87.31f;
	notef[9]=92.50f;
	notef[10]=98.00f;
	notef[11]=103.83f;
	for(int i=12;i<120;i++)
		notef[i]=notef[i-12]*2;

    PortAudioStream *stream;
	Pa_Initialize();
	Pa_OpenDefaultStream(
		&stream,
		0,
		1,
		paInt16,
		44100,
		256,
		0,
		AudioCallback,
		NULL);
	Pa_StartStream(stream);

	// sound effects
	for(int i=0;i<20;i++)
		sprograms[i].length=0;
		
	// player fire
	sprograms[0].speed=50;
	AddSCommand(sprograms[0], 0x05, 30);
	AddSCommand(sprograms[0], 0x01, 42);
	AddSCommand(sprograms[0], 0x03, 40);
	AddSCommand(sprograms[0], 0x00, 0);
	AddSCommand(sprograms[0], 0x01, 45);
	AddSCommand(sprograms[0], 0x03, 25);
	AddSCommand(sprograms[0], 0x00, 0);
	AddSCommand(sprograms[0], 0xFF, 0);

	// block destroyed (channel 1)
	sprograms[1].speed=40;
	AddSCommand(sprograms[1], 0x06, 1);
	AddSCommand(sprograms[1], 0x03, 4);
	AddSCommand(sprograms[1], 0x01, 45);
	AddSCommand(sprograms[1], 0x00, 0);
	AddSCommand(sprograms[1], 0x01, 39);
	AddSCommand(sprograms[1], 0x00, 0);
	AddSCommand(sprograms[1], 0x01, 43);
	AddSCommand(sprograms[1], 0x00, 0);
	AddSCommand(sprograms[1], 0xFF, 0);
	// block destroyed (channel 2)
	sprograms[2].speed=150;
	AddSCommand(sprograms[2], 0x06, 1);
	AddSCommand(sprograms[2], 0x01, 15);
	AddSCommand(sprograms[2], 0x03, 2);
	AddSCommand(sprograms[2], 0x00, 0);
	AddSCommand(sprograms[2], 0xFF, 0);

	// collect
	sprograms[3].speed=30;
	AddSCommand(sprograms[3], 0x05, 50);
	AddSCommand(sprograms[3], 0x01, 45);
	AddSCommand(sprograms[3], 0x00, 0);
	AddSCommand(sprograms[3], 0x01, 49);
	AddSCommand(sprograms[3], 0x00, 0);
	AddSCommand(sprograms[3], 0x01, 53);
	AddSCommand(sprograms[3], 0x00, 0);
	AddSCommand(sprograms[3], 0x05, 40);
	AddSCommand(sprograms[3], 0x01, 45);
	AddSCommand(sprograms[3], 0x00, 0);
	AddSCommand(sprograms[3], 0x01, 51);
	AddSCommand(sprograms[3], 0x00, 0);
	AddSCommand(sprograms[3], 0x01, 47);
	AddSCommand(sprograms[3], 0x00, 0);
	AddSCommand(sprograms[3], 0x01, 55);
	AddSCommand(sprograms[3], 0x00, 0);
	AddSCommand(sprograms[3], 0xFF, 0);

	// music (channel 1)
	sprograms[4].speed=200;
	AddSCommand(sprograms[4], 0x05, 40);
	AddSCommand(sprograms[4], 0x01, 3);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x02, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x01, 15);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x02, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x01, 3);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x01, 11);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x01, 15);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x02, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x01, 3);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x02, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x02, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x02, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x01, 1);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0x02, 0);
	AddSCommand(sprograms[4], 0x00, 0);
	AddSCommand(sprograms[4], 0xFE, 0);

	// gameover
	sprograms[5].speed=150;
	AddSCommand(sprograms[5], 0x05, 30);
	AddSCommand(sprograms[5], 0x03, 6);
	AddSCommand(sprograms[5], 0x01, 50);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x05, 25);
	AddSCommand(sprograms[5], 0x01, 50);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x05, 20);
	AddSCommand(sprograms[5], 0x01, 50);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x05, 15);
	AddSCommand(sprograms[5], 0x01, 50);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x05, 5);
	AddSCommand(sprograms[5], 0x01, 50);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0x00, 0);
	AddSCommand(sprograms[5], 0xFF, 0);

	// thrust channel
	channels[3].mute=true;
	channels[3].noise=true;
	channels[3].noisep=0;
	channels[3].noiseval=0.0f;
	channels[3].volume=0.1f;
	channels[3].phase=0.5f;
	channels[3].slide=0.0f;
	channels[3].pslide=0.0f;
	channels[3].fade=0.0f;
	channels[3].pindex=-1;
	channels[3].active=true;

	// load gfx
	LoadSprite("data/tile1.tga", tiles[0]);
	LoadSprite("data/tiles/greensolo.tga", tiles[1]);
	LoadSprite("data/tiles/greysolo.tga", tiles[2]);
	LoadSprite("data/tiles/greyright.tga", tiles[3]);
	LoadSprite("data/tiles/greyleft.tga", tiles[4]);
	LoadSprite("data/tiles/greycenter.tga", tiles[5]);
	LoadSprite("data/tiles/ice.tga", tiles[6]);
	LoadSprite("data/tiles/redsolo.tga", tiles[7]);
	LoadSprite("data/tiles/grey1.tga", tiles[8]);
	LoadSprite("data/tiles/column1.tga", tiles[9]);
	LoadSprite("data/tiles/collect.tga", tiles[10]);
	LoadSprite("data/tiles/column2.tga", tiles[11]);

	int ed_lasttile=11;
	int ed_curtile=1;
	bool saving=false;
	int ed_curmap=0;
	int ed_tilemap[32*32];
	for(int y=0;y<32;y++)
		for(int x=0;x<32;x++)
			ed_tilemap[y*32+x]=-1;

	LoadSprite("data/star1.tga", spr_stars[0]);
	LoadSprite("data/star2.tga", spr_stars[1]);
	LoadSprite("data/star3.tga", spr_stars[2]);
	LoadSprite("data/star4.tga", spr_stars[3]);

	LoadSprite("data/player2.tga", spr_player[0]);
	LoadSprite("data/player3.tga", spr_player[1]);
	LoadSprite("data/player2fire.tga", spr_player[2]);
	LoadSprite("data/player2fired.tga", spr_player[3]);
	LoadSprite("data/player2walk1.tga", spr_player[4]);
	LoadSprite("data/player2walk2.tga", spr_player[5]);
	LoadSprite("data/player2walk3.tga", spr_player[6]);
	LoadSprite("data/player2walk4.tga", spr_player[7]);

	LoadSprite("data/pshot.tga", spr_pshot);

	LoadSprite("data/playerthrust1.tga", spr_thrust[0]);
	LoadSprite("data/playerthrust2.tga", spr_thrust[1]);
	LoadSprite("data/playerthrust3.tga", spr_thrust[2]);

	LoadSprite("data/blockpoff1.tga", spr_bpoff[0]);
	LoadSprite("data/blockpoff2.tga", spr_bpoff[1]);
	LoadSprite("data/blockpoff3.tga", spr_bpoff[2]);

	Sprite spr_selected;
	LoadSprite("data/selected.tga", spr_selected);

	LoadSprite("data/blip.tga", spr_blip);
	LoadSprite("data/mapgrid.tga", spr_mapgrid);
	LoadSprite("data/win.tga", spr_win);

	// fire animation	
	panim[0][0]=2;
	panim[0][1]=3;
	panim[0][2]=-1;
	// walk animation	
	panim[1][0]=4;
	panim[1][1]=4;
	panim[1][2]=5;
	panim[1][3]=6;
	panim[1][4]=6;
	panim[1][5]=7;
	panim[1][6]=-2;

	Reset();
	
	for(int i=0;i<2000;i++)
	{
		stars[i].x=-512+rnd(2048+1024);
		stars[i].y=-512+rnd(2048+1024);
		stars[i].type=rnd(3);
	}

	PlaySound(4, 7, 0.05f);

/*
	char string[100];
	strcpy(string, "43 54");
	LogPrint("string: \"%s\"", string);
	int v1=0, v2=0;
	LogPrint("before sscanf: v1=%i v2=%i", v1, v2);
	sscanf(string, "%i %i", &v1, &v2);
	LogPrint(" after sscanf: v1=%i v2=%i", v1, v2);
*/
	for(;;)
	{
		if(glfwGetKey(GLFW_KEY_ESC) || !glfwGetWindowParam(GLFW_OPENED))
			break;

		if(gameover && glfwGetKey(GLFW_KEY_ENTER))
		{
			gameover=false;
			Reset();
		}

		bool pwalking=false;
		if(!gameover)
		{
			// move player
			int pspd=12;
			if(player.onfloor)
				pspd=32;
			if(glfwGetKey(GLFW_KEY_RIGHT) && player.reload==0)
			{
				pwalking=true;
				player.dx+=pspd;
				player.flipped=false;
			}
			if(glfwGetKey(GLFW_KEY_LEFT) && player.reload==0)
			{
				pwalking=true;
				player.dx-=pspd;
				player.flipped=true;
			}
			if(!pwalking && player.anim_index==1)
				player.anim_index=-1;
	
			if(glfwGetKey('X'))
			{
				if(player.reload==0)
				{
					for(int i=0;i<20;i++)
						if(!pshots[i].exists)
						{
							pshots[i].exists=true;
							pshots[i].life=75;
							if(!player.flipped)
							{
								pshots[i].x=player.x+8;
								pshots[i].dx=6;
							}
							else
							{
								pshots[i].x=player.x;
								pshots[i].dx=-6;
							}
							pshots[i].y=player.y+4;
							break;
						}
					player.anim_index=0;
					player.anim_pos=0;
					player.anim_delay=0;
					PlaySound(0, 0, 0.1f);
				}
				player.reload=1;
			}
			else
				player.reload=0;
			if(glfwGetKey('Z'))
			{
				player.dy-=24;
				if(player.onfloor) // lift-off, more power
				{
					player.dy-=256;
					// spawn lift-off blast
				}
				player.detach=20;
				player.thrust++;
				if(player.thrust>=6)
					player.thrust=0;
				player.anim_index=-1;
				channels[3].mute=false;
				channels[3].period+=10;
				if(channels[3].period>600)
					channels[3].period=600;
			}
			else
			{
				player.thrust=-1;
				channels[3].mute=true;
				channels[3].period-=10;
				if(channels[3].period<50)
					channels[3].period=50;
			}
			if(player.detach>0)
				player.detach--;
			player.dy+=12;
			SLimit(player.dx, I2F(4));
			SLimit(player.dy, I2F(4));
			player.fx+=player.dx;
			player.fy+=player.dy;
			player.x=F2I(player.fx);
			player.y=F2I(player.fy);
			SetBox(player.box, player.x, player.y, 16, 16);
		}
		
		// animate player
		if(player.anim_index>-1)
		{
			int fi=panim[player.anim_index][player.anim_pos];
			if(fi==-1)
				player.anim_index=-1;
			else if(fi==-2)
			{
				player.anim_pos=0;
				player.frame=panim[player.anim_index][0];
			}
			else
				player.frame=fi;
			player.anim_delay++;
			if(player.anim_delay>4)
			{
				player.anim_delay=0;
				player.anim_pos++;
			}
		}

		// move machines
		for(int i=0;i<200;i++)
			if(machines[i].exists)
			{
				Machine *m=&machines[i];
				m->dx+=rnd(16)-8;
				SLimit(m->dx, 512);
				if(m->x<0) m->dx+=4;
				if(m->x>2048) m->dx-=4;
				m->dy+=rnd(16)-8;
				SLimit(m->dy, 512);
				if(m->y<0) m->dy+=4;
				if(m->y>2048) m->dy-=4;
				
				MachineMove(machines[i]);
				MachineCastPos(machines[i]);
				MachineMakeBox(machines[i], false);
				
				machines[i].collided=false;
			}
		// collide machines/machines
		for(int i=0;i<200;i++)
		{
			if(!machines[i].exists)
				continue;
			bool collide=false;
			for(int j=0;j<200;j++)
			{
				if(i==j)
					continue;
				if(!machines[j].exists)
					continue;
				if(MachinesCollide(machines[i], machines[j], true))
				{
					collide=true;
					MachineMakeBox(machines[j], false);
					machines[i].collided=true;
					machines[j].collided=true;
				}
			}
			if(collide)
				MachineMakeBox(machines[i], false);
		}
		for(int i=0;i<200;i++)
		{
			if(!machines[i].exists)
				continue;
			machines[i].dispfx=machines[i].fx-machines[i].oldfx;
			machines[i].dispfy=machines[i].fy-machines[i].oldfy;
		}		
		if(!gameover)
		{
			// collide player/machines
			bool pcollide=false;
			player.onfloor=false;
			for(int i=0;i<200;i++)
			{
				if(!machines[i].exists)
					continue;
				if(PlayerCollide(machines[i], player.detach==0))
					pcollide=true;
			}
		}

		// move player shots
		for(int i=0;i<20;i++)
			if(pshots[i].exists)
			{
				pshots[i].life--;
				if(pshots[i].life<=0)
					pshots[i].exists=false;
				Box sbox;
				sbox.x=pshots[i].x;
				sbox.y=pshots[i].y+1;
				sbox.w=8;
				sbox.h=2;
				// collide with machines
				for(int j=0;j<200;j++)
				{
					if(!machines[i].exists)
						continue;
					if(BoxMachineIntersect(sbox, machines[j], 1))
					{
						pshots[i].exists=false;
						break;
					}
				}
				pshots[i].x+=pshots[i].dx;
			}

		// update block poffs
		for(int i=0;i<10;i++)
			if(bpoff[i].exists)
			{
				bpoff[i].delay--;
				if(bpoff[i].delay<=0)
				{
					bpoff[i].delay=4;
					bpoff[i].frame++;
					if(bpoff[i].frame>=3)
						bpoff[i].exists=false;
				}
			}

		if(!gameover)
		{
			// camera follows player
			if(camera_x<player.x-160-40) camera_x=player.x-160-40;
			if(camera_x>player.x-160+56) camera_x=player.x-160+56;
			if(camera_y<player.y-120-20) camera_y=player.y-120-20;
			if(camera_y>player.y-120+36) camera_y=player.y-120+36;
		}



		// ---------- editor

		if(editor)
		{
			int mouse_x=0;
			int mouse_y=0;
			glfwGetMousePos(&mouse_x, &mouse_y);
			mouse_x/=2;
			mouse_y/=2;
			
			bool mouseleft=(glfwGetMouseButton(GLFW_MOUSE_BUTTON_LEFT)!=0);
			bool mouseright=(glfwGetMouseButton(GLFW_MOUSE_BUTTON_RIGHT)!=0);
			
			bool tileselect=false;
			if(glfwGetKey(GLFW_KEY_SPACE))
				tileselect=true;
			if(glfwGetKey('C'))
				for(int y=0;y<32;y++)
					for(int x=0;x<32;x++)
						ed_tilemap[y*32+x]=-1;
			if(glfwGetKey('S'))
			{
				if(!saving)
				{
					char fname[32];
					sprintf(fname, "map%i.dat", ed_curmap);
					FILE *file=fopen(fname, "wb");
					fwrite(ed_tilemap, 32*32, sizeof(int), file);
					fclose(file);
					ed_curmap++;
				}
				saving=true;
			}
			else
				saving=false;
				
			// --- render
			glLoadIdentity();
			glClear(GL_COLOR_BUFFER_BIT);

			if(tileselect)
			{
				if(mouseleft)
				{
					ed_curtile=mouse_x/20+mouse_y/20*10;
					if(ed_curtile<0)
						ed_curtile=0;
					if(ed_curtile>ed_lasttile)
						ed_curtile=ed_lasttile;
				}
				int i=0;
				for(int y=0;y<10;y++)
					for(int x=0;x<10;x++)
					{
						if(i>ed_lasttile)
							break;
						DrawSprite(tiles[i], x*20, y*20);
						if(i==ed_curtile)
							DrawSprite(spr_selected, x*20, y*20);
						i++;
					}
			}
			else
			{
				if(mouseleft)
				{
					int index=mouse_y/16*32+mouse_x/16;
					ed_tilemap[index]=ed_curtile;
				}
				if(mouseright)
				{
					int index=mouse_y/16*32+mouse_x/16;
					ed_tilemap[index]=-1;
				}
				for(int y=0;y<32;y++)
					for(int x=0;x<32;x++)
					{
						int ti=ed_tilemap[y*32+x];
						if(ti>-1)
							DrawSprite(tiles[ti], x*16, y*16);
					}
				DrawSprite(spr_selected, mouse_x/16*16, mouse_y/16*16);
			}

			glfwSwapBuffers();
			continue;
		}

		// ---------- editor



		// --- render

		glLoadIdentity();
		glTranslatef((float)-camera_x, (float)-camera_y, 0);

		glClear(GL_COLOR_BUFFER_BIT);

		// draw background
//		if(gameover)
//			glColor4f(1.0f, 0.0f, 0.0f, 1.0f);
		for(int i=0;i<2000;i++)
			DrawSprite(spr_stars[stars[i].type], stars[i].x, stars[i].y);
//		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		// draw shots
		for(int i=0;i<20;i++)
			if(pshots[i].exists)
				DrawSprite(spr_pshot, pshots[i].x, pshots[i].y, pshots[i].dx<0);

		// draw block poffs
		for(int i=0;i<10;i++)
			if(bpoff[i].exists)
				DrawSprite(spr_bpoff[bpoff[i].frame], bpoff[i].x, bpoff[i].y);

		// draw player
		if(!gameover)
		{
			if(player.thrust>-1)
				DrawSprite(spr_thrust[player.thrust/2], player.x-8, player.y-8, player.flipped);
			if(player.onfloor)
			{
				if(player.anim_index==-1)
				{
					if(pwalking)
					{
						player.anim_index=1;
						player.anim_pos=0;
						player.anim_delay=0;
					}
					else
						player.frame=0;
				}
				player.airtime=0;
			}
			else
			{
				player.airtime++;
				if(player.airtime>6)
					player.frame=1;
			}
			DrawSprite(spr_player[player.frame], player.x-8, player.y-8, player.flipped);
		}

		for(int i=0;i<200;i++)
			if(machines[i].exists)
				DrawMachine(machines[i], false);

		// map
		glPushMatrix();
		glLoadIdentity();
		glColor4f(1.0f, 1.0f, 1.0f, 0.5f);
		for(int y=0;y<12;y++)
			for(int x=0;x<12;x++)
				DrawSprite(spr_mapgrid, x*8, y*8);
		glColor4f(0.5f, 1.0f, 1.0f, 1.0f);
		for(int i=0;i<200;i++)
			if(machines[i].exists)
				DrawMachine(machines[i], true);
		glColor4f(1.0f, 1.0f, 0.0f, 1.0f);
		DrawSprite(spr_blip, 16+player.x/32, 16+player.y/32);

		if(gameover)
		{
			Box gbox;
			gbox.x=0;
			gbox.y=0;
			gbox.w=320;
			gbox.h=240;
			glColor4f(0.5f, 0.0f, 0.0f, 0.5f);
			DrawBox(gbox);
			death_alpha-=0.05f;
			if(death_alpha<0.0f)
				death_alpha=0.0f;
			glColor4f(1.0f, 1.0f, 1.0f, death_alpha);
			DrawBox(gbox);
		}
		if(collectibles<=0)
		{
			glColor4f(0.5f, 1.0f, 0.5f, 1.0f);
			DrawSprite(spr_win, 160-64, 120-32);
		}
		glPopMatrix();

		glColor4f(1.0f, 1.0f, 1.0f, 1.0f);

		glfwSwapBuffers();
	}

	Pa_StopStream(stream);
	Pa_CloseStream(stream);
	Pa_Terminate();

	return 0;
}

